﻿using NCalc;
using QPR.QPRSuite.AcceleratorTools;
using System;
using System.Collections.Generic;

namespace QPR.QPRSuite.NCalc {

    public abstract class CustomFunctionBase {

        protected IContextManager context;
        protected int requiredNumberOfParameters;
        protected object[] parameters;
        public static Dictionary<string, string> TYPE_CONVERSIONS = new Dictionary<string, string>();

        static CustomFunctionBase() {
            CustomFunctionBase.TYPE_CONVERSIONS.Add("int32", "int");
            CustomFunctionBase.TYPE_CONVERSIONS.Add("int64", "int");
        }

        public CustomFunctionBase(FunctionArgs arguments, IContextManager context, int requiredNumberOfParameters, string[] parameterTypes = null) {
            this.context = context;
            this.requiredNumberOfParameters = requiredNumberOfParameters;

            this.parameters = new object[arguments.Parameters.Length];
            for (int i = 0; i < arguments.Parameters.Length; i++) {
                parameters[i] = arguments.Parameters[i].Evaluate();
            }

            if (this.parameters.Length < requiredNumberOfParameters) throw new Exception("Function \"" + this.GetType().Name + "\" doesn't have enough parameters.");

            if (parameterTypes != null) {
                for (int i = 0; i < parameterTypes.Length; i++) {
                    if (i < this.parameters.Length && this.parameters[i] != null) {
                        string parameterType = this.parameters[i].GetType().Name.ToLower();
                        Boolean isArray = this.parameters[i].GetType().IsArray;
                        if ((!isArray && CustomFunctionBase.convertToBasicTypes(parameterType) != parameterTypes[i].ToLower() && parameterTypes[i].ToLower() != "object") || (isArray && CustomFunctionBase.convertToBasicTypes(parameterType.Replace("[]", "")) != parameterTypes[i].ToLower().Replace("[]", "") && parameterTypes[i].ToLower() != "object[]")) {
                            throw new Exception("In function \"" + this.GetType().Name + "\" " + (i + 1) + ". parameter type must be \"" + parameterTypes[i] + "\". Type \"" + parameterType + "\" was encountered.");
                        }
                    }
                }

                if (parameters.Length < parameterTypes.Length) {
                    object[] newParameters = new object[parameterTypes.Length];
                    for (int i = 0; i < parameterTypes.Length; i++) {
                        if (parameters.Length > i) {
                            newParameters[i] = parameters[i];
                        } else {
                            newParameters[i] = null;
                        }
                    }
                    this.parameters = newParameters;
                }
            }
        }

        public static string convertToBasicTypes(string inputType) {
            if (CustomFunctionBase.TYPE_CONVERSIONS.ContainsKey(inputType)) return CustomFunctionBase.TYPE_CONVERSIONS[inputType];
            return inputType;
        }

        public abstract object evaluate();

    }
}