﻿using NCalc;
using QPR.QPRSuite.AcceleratorTools;
using System;
using System.Collections.Generic;

namespace QPR.QPRSuite.NCalc {
    public class addcolumn : CustomFunctionBase {

        public addcolumn(FunctionArgs args, IContextManager context)
            : base(args, context, 3, new string[] { "dataset", "string", "string", "boolean" }) {
        }

        public override object evaluate() {
            DataSet oldDataSet = (DataSet)this.parameters[0];
            if (oldDataSet == null) return null;
            List<string> newColumns = new List<string>(oldDataSet.getColumns());
            string newColumnName = (string)this.parameters[1];
            string expression = (string)this.parameters[2];
            bool compareMode = false;
            if (this.parameters.Length > 3 && this.parameters[3] != null) compareMode = (bool)this.parameters[3];
            if (string.IsNullOrEmpty(newColumnName)) throw new Exception("Column name is not allowed to be empty in AddColumn");
            if (oldDataSet.getColumnNumber(newColumnName) > -1) throw new Exception("Dataset already contains column \"" + newColumnName + "\"");
            newColumns.Add(newColumnName);
            DataSet newDataSet = new DataSet(newColumns.ToArray());
            for (int i = 0; i < oldDataSet.rowsCount(); i++) {
                object[] oldRowData = oldDataSet.getRow(i).getData();
                object[] newRowData = new object[newColumns.Count];
                oldRowData.CopyTo(newRowData, 0);
                Dictionary<string, object> arguments = new Dictionary<string, object>();
                if (compareMode) {
                    if (i == 0) {
                        for (int j = 0; j < newDataSet.getColumns().Length; j++) {
                            arguments.Add(newDataSet.getColumnName(j) + "_previous", null);
                        }
                    } else {
                        object[] previousRow = newDataSet.getRow(i - 1).getData();
                        for (int j = 0; j < previousRow.Length; j++) {
                            arguments.Add(newDataSet.getColumnName(j) + "_previous", previousRow[j]);
                        }
                    }
                    for (int j = 0; j < oldRowData.Length; j++) {
                        arguments.Add(oldDataSet.getColumnName(j) + "_current", oldRowData[j]);
                    }
                    if (i == oldDataSet.rowsCount() - 1) {
                        for (int j = 0; j < oldDataSet.getColumns().Length; j++) {
                            arguments.Add(newDataSet.getColumnName(j) + "_next", null);
                        }
                    } else {
                        object[] nextRow = oldDataSet.getRow(i + 1).getData();
                        for (int j = 0; j < nextRow.Length; j++) {
                            arguments.Add(oldDataSet.getColumnName(j) + "_next", nextRow[j]);
                        }
                    }
                } else {
                    for (int j = 0; j < oldRowData.Length; j++) {
                        arguments.Add(oldDataSet.getColumnName(j), oldRowData[j]);
                    }
                }
                for (int j = 4; j < this.parameters.Length; j++) {
                    if (!arguments.ContainsKey("argument" + (j - 3))) arguments.Add("argument" + (j - 3), this.parameters[j]);
                }
                if (!arguments.ContainsKey("rowordernumber")) arguments.Add("rowordernumber", i);
                try {
                    newRowData[oldRowData.Length] = this.context.getExpressionEngine().evaluateExpressionAsObject(expression, arguments);
                } catch (Exception e) {
                    throw new Exception("Evaluating expression \"" + expression + "\" failed. Row number is " + oldRowData.Length + ". This error occured in AddColumn function.", e);
                }
                newDataSet.addRow(newRowData);
            }
            return newDataSet;
        }

    }
}