﻿using NCalc;
using QPR.QPRSuite.AcceleratorTools;
using QPR.QPRSuite.AcceleratorTools.QPRWebService;
using System;
using System.Collections.Generic;

namespace QPR.QPRSuite.NCalc {
    public class diagrampath : CustomFunctionBase {

        public diagrampath(FunctionArgs args, IContextManager context)
            : base(args, context, 4, new string[] { "string", "string", "string", "string", "string" }) {
        }

        public override object evaluate() {
            string objectId = (string)this.parameters[0];
            if (string.IsNullOrEmpty(objectId)) return new List<string>().ToArray();
            string attribute = (string)this.parameters[1];
            if (string.IsNullOrEmpty(attribute)) throw new Exception("Attribute cannot be empty in DiagramPath");
            string parentRelation = (string)this.parameters[2];
            if (string.IsNullOrEmpty(parentRelation)) throw new Exception("Parent relation cannot be empty in DiagramPath");
            string separator = (string)this.parameters[3];
            if (separator == null) separator = "";
            string options = (string)this.parameters[4];
            if (options == null) options = "";
            List<string> diagramPaths = this.processDiagramPathStep(objectId, parentRelation, attribute, separator, options);
            diagramPaths.Sort();
            return diagramPaths.ToArray();
        }

        private List<string> processDiagramPathStep(string objectId, string parentRelation, string attribute, string separator, string options) {
            List<string> paths = new List<string>();
            ResultSet results = this.context.getWebServiceClient().queryObjects("[" + objectId + "]." + parentRelation, "typename<>\"Model\"", "id", attribute, options);
            if (results == null || results.ChildElements == null || results.ChildElements.Length == 0) return paths;
            for (int i = 0; i < results.ChildElements.Length; i++) {
                string parentId = ((ResultObject)results.ChildElements[i]).Id;
                List<string> diagramPaths = this.processDiagramPathStep(parentId, parentRelation, attribute, separator, options);
                string attributeValue = "";
                if (((ResultObject)results.ChildElements[i]).ChildElements != null && ((ResultObject)results.ChildElements[i]).ChildElements.Length > 0) {
                    attributeValue = ((ValueElement)((ResultObject)results.ChildElements[i]).ChildElements[0]).Value;
                }
                if (diagramPaths.Count == 0) {
                    paths.Add(attributeValue);
                } else {
                    for (int j = 0; j < diagramPaths.Count; j++) {
                        paths.Add(diagramPaths[j] + separator + attributeValue);
                    }
                }
            }
            return paths;
        }

    }
}