﻿using NCalc;
using QPR.QPRSuite.AcceleratorTools;
using System;
using System.Collections.Generic;

namespace QPR.QPRSuite.NCalc {
    public class executerecursion : CustomFunctionBase {

        private List<object> traversedItems = new List<object>();
        private Dictionary<string, object> additionalArguments = new Dictionary<string, object>();
        private string returnExpression = null;
        private string recursionExpression = null;
        private bool excludeTraversed = false;

        public executerecursion(FunctionArgs args, IContextManager context)
            : base(args, context, 4, new string[] { "string", "string", "object", "boolean" }) {
        }

        public override object evaluate() {
            this.returnExpression = (string)this.parameters[0];
            this.recursionExpression = (string)this.parameters[1];
            object initialValue = this.parameters[2];
            this.excludeTraversed = (bool)this.parameters[3];
            for (int i = 4; i < this.parameters.Length; i++) {
                additionalArguments["argument" + (i - 3)] = this.parameters[i];
            }

            return this.executeRecursionStep(initialValue);
        }

        private object executeRecursionStep(object initialValue) {
            List<object> subReturnValues = new List<object>();
            Dictionary<string, object> arguments = new Dictionary<string, object>();
            arguments.Add("currentrecursionstep", initialValue);
            foreach (string key in this.additionalArguments.Keys) {
                arguments.Add(key, additionalArguments[key]);
            }
            object[] subObjectsSet = (object[])this.context.getExpressionEngine().evaluateExpressionAsObject(this.recursionExpression, arguments);
            for (int i = 0; i < subObjectsSet.Length; i++) {
                if (this.excludeTraversed && this.traversedItems.Contains(subObjectsSet[i])) continue;
                this.traversedItems.Add(subObjectsSet[i]);
                Dictionary<string, object> subArguments = new Dictionary<string, object>();
                List<string> expressionText = new List<string>();
                subReturnValues.Add(this.executeRecursionStep(subObjectsSet[i]));
            }
            arguments = new Dictionary<string, object>();
            arguments.Add("recursionresult", subReturnValues.ToArray());
            arguments.Add("currentrecursionstep", initialValue);
            foreach (string key in this.additionalArguments.Keys) {
                arguments.Add(key, additionalArguments[key]);
            }
            return this.context.getExpressionEngine().evaluateExpressionAsObject(this.returnExpression, arguments);
        }

    }
}