﻿using NCalc;
using QPR.QPRSuite.AcceleratorTools;
using QPR.QPRSuite.AcceleratorTools.QPRWebService;
using System;
using System.Collections.Generic;
using System.Globalization;

namespace QPR.QPRSuite.NCalc {

    public class getattribute : CustomFunctionBase {

        private string objectid;
        private string attribute;
        private string options;
        private string followrelations = null;
        private string separator = "";
        private string sortby = "";
        public static string[] WEB_SERVICE_SEPARATOR = { "," };
        public static char[] INPUT_SEPARATOR = { ',' };

        public getattribute(FunctionArgs args, IContextManager context)
            : base(args, context, 3, new string[] { "string", "string", "string", "string", "string", "string"}) {
            this.objectid = Convert.ToString(args.Parameters[0].Evaluate(), CultureInfo.InvariantCulture);
            this.attribute = Convert.ToString(args.Parameters[1].Evaluate(), CultureInfo.InvariantCulture);
            this.options = Convert.ToString(args.Parameters[2].Evaluate(), CultureInfo.InvariantCulture);
            if (args.Parameters.Length > 3) this.followrelations = Convert.ToString(args.Parameters[3].Evaluate(), CultureInfo.InvariantCulture);
            if (args.Parameters.Length > 4) this.separator = Convert.ToString(args.Parameters[4].Evaluate(), CultureInfo.InvariantCulture);
            if (args.Parameters.Length > 5) this.sortby = Convert.ToString(args.Parameters[5].Evaluate(), CultureInfo.InvariantCulture);
        }

        public override object evaluate() {
            string attributeValue = "";
            if ((followrelations == null || followrelations == "") && !objectid.Contains(",")) {
                attributeValue = this.context.getWebServiceClient().getAttributeAsString(objectid, attribute, options);

            } else { // if "followrelations" is in use or Object contains multiple id's
                String[] targetObjectsArray = null;
                if (followrelations != null && followrelations != "") {
                    string[] relations = followrelations.Split(INPUT_SEPARATOR, StringSplitOptions.RemoveEmptyEntries);
                    HashSet<string> sourceObjects = new HashSet<string>();
                    sourceObjects = new HashSet<string>(objectid.Split(INPUT_SEPARATOR));
                    HashSet<string> targetObjects = sourceObjects;
                    foreach (string relationName in relations) {
                        if (sourceObjects.Count == 0) return "";
                        targetObjects = new HashSet<string>();
                        String[] sourceObjectsArray = new String[sourceObjects.Count];
                        sourceObjects.CopyTo(sourceObjectsArray);
                        ResultSet results = this.context.getWebServiceClient().queryObjects("[" + String.Join(WEB_SERVICE_SEPARATOR[0], sourceObjectsArray) + "]", "", sortby, "related(relation=\"" + relationName + "\")", options);
                        for (int i = 0; results != null && results.ChildElements != null && i < results.ChildElements.Length; i++) {
                            if (results.ChildElements[i].ChildElements != null && results.ChildElements[i].ChildElements.Length > 0 && ((ValueElement)((ResultObject)results.ChildElements[i]).ChildElements[0]).Value != "") {
                                string[] targetObjectsPartArray = ((ValueElement)((ResultObject)results.ChildElements[i]).ChildElements[0]).Value.Split(WEB_SERVICE_SEPARATOR, StringSplitOptions.RemoveEmptyEntries);
                                foreach (string targetObject in targetObjectsPartArray) {
                                    targetObjects.Add(targetObject);
                                }
                            }
                        }
                        sourceObjects = targetObjects;
                    }

                    targetObjectsArray = new String[targetObjects.Count];
                    targetObjects.CopyTo(targetObjectsArray);
                } else { //"followrelations" not in use
                    targetObjectsArray = objectid.Split(INPUT_SEPARATOR, StringSplitOptions.RemoveEmptyEntries);
                }
                ResultSet endResults = this.context.getWebServiceClient().queryObjects("[" + String.Join(WEB_SERVICE_SEPARATOR[0], targetObjectsArray) + "]", "", sortby, attribute, options);
                List<string> attributeValues = new List<string>();
                for (int i = 0; endResults != null && endResults.ChildElements != null && i < endResults.ChildElements.Length; i++) {
                    if (endResults.ChildElements[i].ChildElements != null && endResults.ChildElements[i].ChildElements.Length > 0 && ((ValueElement)((ResultObject)endResults.ChildElements[i]).ChildElements[0]).Value != "") {
                        string returnValue = ((ValueElement)((ResultObject)endResults.ChildElements[i]).ChildElements[0]).Value;
                        if (returnValue != null && returnValue != "") attributeValues.Add(returnValue);
                    }
                }
                attributeValue = String.Join(separator, attributeValues);
            }
            if (attributeValue == null) attributeValue = "";
            return attributeValue;
        }

    }
}