﻿using NCalc;
using QPR.QPRSuite.AcceleratorTools;
using System;
using System.Collections.Generic;
using System.Linq;

namespace QPR.QPRSuite.NCalc {
    public class groupby : CustomFunctionBase {

        public groupby(FunctionArgs args, IContextManager context)
            : base(args, context, 2, new string[] { "DataSet", "String" }) {
        }

        public override object evaluate() {
            DataSet oldDataSet = (DataSet)this.parameters[0];
            string groupAttributes = (string)this.parameters[1];
            if (groupAttributes == null) groupAttributes = "";
            string[] groupAttributesArray = groupAttributes.Split(',');
            string aggregatedAttributes = (string)this.parameters[2];
            if (aggregatedAttributes == null) aggregatedAttributes = "";
            string[] aggregatedAttributesArray = aggregatedAttributes.Split(',');
            if (aggregatedAttributesArray.Length != this.parameters.Length - 3) throw new Exception("Number of aggregate columns don't match number of provided formulas in GroupBy");
            string[] groupingFormulas = new string[aggregatedAttributesArray.Length];
            for (int i = 3; i < this.parameters.Length; i++) {
                groupingFormulas[i - 3] = (string)this.parameters[i];
            }
            DataSet sortedDataset = oldDataSet.copy();
            if (groupAttributesArray.Length > 0) sortedDataset.sort(new DatasetComparer(groupAttributes));
            GroupByRowEqualityComparer groupByRowEqualityComparer = new GroupByRowEqualityComparer(groupAttributes);
            DataSet resultDataset = new DataSet(groupAttributesArray.Concat(aggregatedAttributesArray).ToArray<string>());
            int groupStartIndex = 0;
            for (int i = 0; i < sortedDataset.rowsCount(); i++) {
                if (i < sortedDataset.rowsCount() - 1 && !groupByRowEqualityComparer.Equals(sortedDataset.getRow(i), sortedDataset.getRow(i + 1)) || i == sortedDataset.rowsCount() - 1) {
                    List<object>[] groupingData = new List<object>[sortedDataset.columnsCount()];
                    for (int columnIndex = 0; columnIndex < sortedDataset.columnsCount(); columnIndex++) {
                        groupingData[columnIndex] = new List<object>();
                    }
                    for (int j = groupStartIndex; j <= i; j++) {
                        for (int columnIndex = 0; columnIndex < sortedDataset.columnsCount(); columnIndex++) {
                            groupingData[columnIndex].Add(sortedDataset.getCell(columnIndex, j));
                        }
                    }
                    Dictionary<string, object> arguments = new Dictionary<string, object>();
                    for (int columnIndex = 0; columnIndex < sortedDataset.columnsCount(); columnIndex++) {
                        arguments.Add(sortedDataset.getColumnName(columnIndex), groupingData[columnIndex].ToArray());
                    }
                    object[] rowData = new object[groupAttributesArray.Length + aggregatedAttributesArray.Length];

                    for (int j = 0; j < groupAttributesArray.Length; j++) {
                        rowData[j] = sortedDataset.getCell(groupAttributesArray[j], i);
                    }
                    for (int j = groupAttributesArray.Length; j < (aggregatedAttributesArray.Length + groupAttributesArray.Length); j++) {
                        rowData[j] = context.getExpressionEngine().evaluateExpressionAsObject(groupingFormulas[j - groupAttributesArray.Length], arguments);
                    }
                    resultDataset.addRow(rowData);
                    groupStartIndex = i + 1;
                }
            }
            return resultDataset;
        }

    }

    public class GroupByRowEqualityComparer : IEqualityComparer<DataSetRow> {

        string[] comparisonAttributesArray;

        public GroupByRowEqualityComparer(string comparisonAttributes) {
            comparisonAttributesArray = comparisonAttributes.Split(',');
        }

        public bool Equals(DataSetRow x, DataSetRow y) {
            for (int i = 0; i < comparisonAttributesArray.Length; i++) {
                if (DataSetRow.compareObjects(x.getCell(comparisonAttributesArray[i]), y.getCell(comparisonAttributesArray[i])) != 0) return false;
            }
            return true;
        }

        public int GetHashCode(DataSetRow datasetRow) {
            return datasetRow.GetHashCode();
        }

    }

}