﻿using NCalc;
using QPR.QPRSuite.AcceleratorTools;
using System;
using System.Globalization;
using System.Net;

namespace QPR.QPRSuite.NCalc {

    public class httpfilemediatype : CustomFunctionBase {

        private string url;
        private string alternateUrl = null;

        public httpfilemediatype(FunctionArgs args, IContextManager context)
            : base(args, context, 1, new string[] { "string", "string" }) {
            this.url = Convert.ToString(this.parameters[0], CultureInfo.InvariantCulture);
            if (this.parameters.Length > 1) {
                this.alternateUrl = (string)this.parameters[1];
            }
        }

        public override object evaluate() {
            if (url == null || url == "") return "";
            try {
                WebRequest request = WebRequest.Create(url);
                request.Credentials = CredentialCache.DefaultCredentials;
                WebResponse response = null;
                try {
                    response = request.GetResponse();
                } catch (Exception e) {
                    if (!e.Message.Contains("404") || String.IsNullOrEmpty(this.alternateUrl)) throw e;
                    request = WebRequest.Create(alternateUrl);
                    request.Credentials = CredentialCache.DefaultCredentials;
                    response = request.GetResponse();
                }
                string contentType = response.ContentType;
                response.Close();
                return contentType;
            } catch (Exception e) {
                this.context.getLogger().writeLog("Getting file data type using http failed. Url=\"" + url + "\", " + e.Message, LogLevel.information);
            }
            return "";
        }

    }
}