﻿using NCalc;
using QPR.QPRSuite.AcceleratorTools;
using QPR.QPRSuite.AcceleratorTools.QPRWebService;
using System;
using System.Collections.Generic;

namespace QPR.QPRSuite.NCalc {
    public class metricsmeasurevalues : CustomFunctionBase {

        public metricsmeasurevalues(FunctionArgs args, IContextManager context)
            : base(args, context, 4, new string[] { "dataset", "string", "string", "string", "string", "string" }) {
        }

        public override object evaluate() {
            DataSet dataSet = (DataSet)this.parameters[0];
            if (dataSet == null) return null;
            if (dataSet.rowsCount() == 0) return dataSet;
            string elementIdColumn = (string)this.parameters[1];
            if (string.IsNullOrEmpty(elementIdColumn)) throw new Exception("Element id column is empty in MetricsMeasureData function");
            string seriesSymbolColumn = (string)this.parameters[2];
            if (string.IsNullOrEmpty(seriesSymbolColumn)) throw new Exception("Series symbol column is empty in MetricsMeasureData function");
            string attributeName = (string)this.parameters[3];
            if (string.IsNullOrEmpty(attributeName)) throw new Exception("Attribute name column is empty in MetricsMeasureData function");
            string startPeriod = null;
            if (this.parameters.Length > 3) startPeriod = (string)this.parameters[4];
            string endPeriod = null;
            if (this.parameters.Length > 4) endPeriod = (string)this.parameters[5];
            int elementIdColumnNumber = dataSet.getColumnNumber(elementIdColumn);
            if (elementIdColumnNumber == -1) throw new Exception("Element id column \"" + elementIdColumn + "\" was not found in dataset in MetricsMeasureData function");
            int seriesSymbolColumnNumber = dataSet.getColumnNumber(seriesSymbolColumn);
            if (seriesSymbolColumnNumber == -1) throw new Exception("Series symbol column \"" + seriesSymbolColumn + "\" was not found in dataset in MetricsMeasureData function");

            Dictionary<string, string> valuesDictionary = new Dictionary<string, string>();
            Dictionary<string, string> measureValuesettings = new Dictionary<string, string>();
            Dictionary<string, string> measurePeriodlevels = new Dictionary<string, string>();
            List<string> objectIds = new List<string>();
            for (int i = 0; i < dataSet.rowsCount(); i++) {
                objectIds.Add((string)dataSet.getCell(elementIdColumnNumber, i));
            }
            ResultSet values = this.context.getWebServiceClient().queryObjects("[" + String.Join(",", objectIds.ToArray()) + "]", "", "", "values,valuesettingid,periodlevelid", "");
            if (values != null && values.ChildElements != null && values.ChildElements.Length > 0) {
                for (int i = 0; i < values.ChildElements.Length; i++) {
                    ResultObject resultObject = (ResultObject)values.ChildElements[i];
                    if (resultObject.ChildElements != null && resultObject.ChildElements.Length > 0) {
                        string valueSettingsId = this.getValueElementAttribute(resultObject, "valuesettingid");
                        if (!string.IsNullOrEmpty(valueSettingsId)) measureValuesettings.Add(resultObject.Id, valueSettingsId);
                        string periodLevelId = this.getValueElementAttribute(resultObject, "periodlevelid");
                        if (!string.IsNullOrEmpty(periodLevelId)) measurePeriodlevels.Add(resultObject.Id, periodLevelId);
                        if (resultObject.ChildElements.Length > 2 && resultObject.ChildElements[2].ChildElements != null) {
                            for (int j = 0; j < ((ValueElement)resultObject.ChildElements[2]).ChildElements.Length; j++) {
                                Element element = (Element)((ValueElement)resultObject.ChildElements[2]).ChildElements[j];
                                string seriesId = this.getElementAttribute(element, "id");
                                for (int k = 0; element.ChildElements != null && k < element.ChildElements.Length; k++) {
                                    Element subElement = (Element)element.ChildElements[k];
                                    string periodId = this.getElementAttribute(subElement, "periodid");
                                    string value = this.getElementAttribute(subElement, attributeName);
                                    valuesDictionary.Add(resultObject.Id + "|" + seriesId + "|" + periodId, value);
                                }
                            }
                        }
                    }
                }
            }

            Dictionary<string, string> series = new Dictionary<string, string>();
            List<string> queriedValuesettings = new List<string>();
            foreach (string key in measureValuesettings.Keys) {
                if (!queriedValuesettings.Contains(measureValuesettings[key])) {
                    ResultSet seriesResult = this.context.getWebServiceClient().queryObjects("[" + measureValuesettings[key] + "].series", "", "", "symbol", "");
                    if (seriesResult != null && seriesResult.ChildElements != null && seriesResult.ChildElements.Length > 0) {
                        for (int i = 0; i < seriesResult.ChildElements.Length; i++) {
                            if (seriesResult.ChildElements[i].ChildElements != null && seriesResult.ChildElements[i].ChildElements.Length > 0) {
                                string seriesSymbol = ((ValueElement)((ResultObject)seriesResult.ChildElements[i]).ChildElements[0]).Value;
                                series.Add(measureValuesettings[key] + "|" + seriesSymbol, ((ResultObject)seriesResult.ChildElements[i]).Id);
                            }
                        }
                    }
                }
                queriedValuesettings.Add(measureValuesettings[key]);
            }

            Dictionary<string, List<string>> periodsById = new Dictionary<string, List<string>>();
            Dictionary<string, List<string>> periodsByName = new Dictionary<string, List<string>>();
            Dictionary<string, List<string>> periodsByStartDate = new Dictionary<string, List<string>>();
            Dictionary<string, List<string>> periodsByEndDate = new Dictionary<string, List<string>>();
            List<string> queriedPeriodlevels = new List<string>();
            foreach (string key in measurePeriodlevels.Keys) {
                if (!queriedPeriodlevels.Contains(measurePeriodlevels[key])) {
                    List<string> periodIdList = new List<string>();
                    List<string> periodNameList = new List<string>();
                    List<string> periodStartDateList = new List<string>();
                    List<string> periodEndDateList = new List<string>();
                    ResultSet periodResult = this.context.getWebServiceClient().queryObjects("[" + measurePeriodlevels[key] + "].periods", "", "startdate", "name,startdate,enddate", "");
                    if (periodResult != null && periodResult.ChildElements != null && periodResult.ChildElements.Length > 0) {
                        for (int i = 0; i < periodResult.ChildElements.Length; i++) {
                            if (periodResult.ChildElements[i].ChildElements != null && periodResult.ChildElements[i].ChildElements.Length > 2) {
                                periodIdList.Add(((ResultObject)periodResult.ChildElements[i]).Id);
                                periodNameList.Add(((ValueElement)((ResultObject)periodResult.ChildElements[i]).ChildElements[0]).Value);
                                periodStartDateList.Add(((ValueElement)((ResultObject)periodResult.ChildElements[i]).ChildElements[1]).Value);
                                periodEndDateList.Add(((ValueElement)((ResultObject)periodResult.ChildElements[i]).ChildElements[2]).Value);
                            }
                        }
                    }
                    periodsById.Add(measurePeriodlevels[key], periodIdList);
                    periodsByName.Add(measurePeriodlevels[key], periodNameList);
                    periodsByStartDate.Add(measurePeriodlevels[key], periodStartDateList);
                    periodsByEndDate.Add(measurePeriodlevels[key], periodEndDateList);
                }
                queriedPeriodlevels.Add(measurePeriodlevels[key]);
            }

            Dictionary<int, Dictionary<string, string>> measureValues = new Dictionary<int, Dictionary<string, string>>();
            List<string> periodOrder = new List<string>();
            for (int i = 0; i < dataSet.rowsCount(); i++) {
                measureValues.Add(i, new Dictionary<string, string>());
                string objectId = Convert.ToString(dataSet.getCell(elementIdColumnNumber, i));
                string seriesSymbol = Convert.ToString(dataSet.getCell(seriesSymbolColumnNumber, i));
                if (!measurePeriodlevels.ContainsKey(objectId)) continue;
                string periodLevelId = measurePeriodlevels[objectId];
                List<string> periodIdsToGetValues = new List<string>();
                List<string> periodNamesToGetValues = new List<string>();
                if (true || string.IsNullOrEmpty(startPeriod) && string.IsNullOrEmpty(startPeriod)) { //TODO
                    for (int j = 0; j < periodsById[periodLevelId].Count; j++) {
                        periodIdsToGetValues.Add(periodsById[periodLevelId][j]);
                        periodNamesToGetValues.Add(periodsByName[periodLevelId][j]);
                        string periodName = periodsByName[periodLevelId][j];
                        if (!periodOrder.Contains(periodName)) periodOrder.Add(periodsByName[periodLevelId][j]);
                    }
                } else {
                    //...
                }
                for (int j = 0; j < periodIdsToGetValues.Count; j++) {
                    if (!measureValuesettings.ContainsKey(objectId)) continue;
                    string valuesettingsId = measureValuesettings[objectId];
                    if (!series.ContainsKey(valuesettingsId + "|" + seriesSymbol)) continue;
                    string seriesId = series[valuesettingsId + "|" + seriesSymbol];
                    string valueIdentifier = objectId + "|" + seriesId + "|" + periodIdsToGetValues[j];
                    if (!valuesDictionary.ContainsKey(valueIdentifier)) continue;
                    measureValues[i].Add(periodNamesToGetValues[j], valuesDictionary[valueIdentifier]);
                }
            }

            string[] columns = new string[dataSet.getColumns().Length + periodOrder.Count];
            for (int i = 0; i < dataSet.getColumns().Length; i++) {
                columns[i] = dataSet.getColumns()[i];
            }
            for (int i = 0; i < periodOrder.Count; i++) {
                columns[i + dataSet.getColumns().Length] = periodOrder[i];
            }
            DataSet resultDataSet = new DataSet(columns);
            for (int i = 0; i < dataSet.rowsCount(); i++) {
                object[] rowData = new object[columns.Length];
                for (int j = 0; j < dataSet.getColumns().Length; j++) {
                    rowData[j] = dataSet.getCell(j, i);
                }
                for (int j = 0; j < periodOrder.Count; j++) {
                    if (measureValues[i].ContainsKey(periodOrder[j])) {
                        rowData[j + dataSet.getColumns().Length] = measureValues[i][periodOrder[j]];
                    } else {
                        rowData[j + dataSet.getColumns().Length] = null;
                    }
                }
                resultDataSet.addRow(rowData);
            }

            return resultDataSet;
        }

        private string getElementAttribute(Element element, string attributeName) {
            if (element == null) return null;
            foreach (ValueElement attribute in element.Attributes) {
                if (attribute.Name.ToLower() == attributeName.ToLower()) return attribute.Value;
            }
            return null;
        }

        private string getValueElementAttribute(ResultObject element, string attributeName) {
            if (element == null) return null;
            foreach (ValueElement attribute in element.ChildElements) {
                if (attribute.Name.ToLower() == attributeName.ToLower()) return attribute.Value;
            }
            return null;
        }
    }
}