﻿using NCalc;
using QPR.QPRSuite.AcceleratorTools;
using System.Collections.Generic;

namespace QPR.QPRSuite.NCalc {

    public class sortby : CustomFunctionBase {

        public sortby(FunctionArgs args, IContextManager context)
            : base(args, context, 2, new string[] { "dataset", "string" }) {
        }

        public override object evaluate() {
            DataSet dataSet = (DataSet)this.parameters[0];
            if (dataSet == null) return null;
            string sortBy = (string)this.parameters[1];
            DatasetComparer comparer = new DatasetComparer(sortBy);
            DataSet newDataset = dataSet.copy();
            newDataset.sort(comparer);
            return newDataset;
        }

    }

    public class DatasetComparer : IComparer<DataSetRow> {

        private string[] sortbyAttributes;
        private string[] sortbyDirection;

        public DatasetComparer(string sortBy) {
            string[] sortByData = sortBy.Split(',');
            this.sortbyAttributes = new string[sortByData.Length];
            this.sortbyDirection = new string[sortByData.Length];
            for (int i = 0; i < sortByData.Length; i++) {
                string item = sortByData[i].Trim();
                if (item.ToLower().EndsWith(" desc") && item.Length > 5) {
                    this.sortbyDirection[i] = "desc";
                    this.sortbyAttributes[i] = item.Substring(0, item.Length - 5).Trim();
                } else {
                    sortbyDirection[i] = "asc";
                    if (item.ToLower().EndsWith(" asc")) {
                        this.sortbyAttributes[i] = item.Substring(0, item.Length - 4).Trim();
                    } else {
                        this.sortbyAttributes[i] = item;
                    }
                }
            }
        }

        public int Compare(DataSetRow datasetRow1, DataSetRow datasetRow2) {
            for (int i = 0; i < sortbyAttributes.Length; i++) {
                int comparison = DataSetRow.compareObjects(datasetRow1.getCell(sortbyAttributes[i]), datasetRow2.getCell(sortbyAttributes[i]));
                if (comparison != 0) return (this.sortbyDirection[i] == "asc" ? comparison : -comparison);
            }
            return 0;
        }

    }
}