﻿using NCalc;
using QPR.QPRSuite.AcceleratorTools;
using System;
using System.Collections.Generic;
using System.IO;
using System.Xml;
using System.Xml.Linq;
using System.Xml.Schema;
using System.Linq;

namespace QPR.QPRSuite.NCalc {
    public class xmldocument : CustomFunctionBase {

        public static XNamespace CONFIGURATION_XML_NAMESPACE = "http://www.qpr.com/NCalcExpressionLanguage";
        private List<string> xmlValidationErrors = new List<string>();

        public xmldocument(FunctionArgs args, IContextManager context)
            : base(args, context, 2, new string[] { "string", "string", "string" }) {
        }

        public override object evaluate() {
            string xmlData = (string)this.parameters[0];
            if (string.IsNullOrEmpty(xmlData)) throw new Exception("XML data was empty in \"XmlDocument\" function");
            string xmlSchema = (string)this.parameters[1];
            if (string.IsNullOrEmpty(xmlSchema)) throw new Exception("XML schema was empty in \"XmlDocument\" function");
            string customErrorMessage = null;
            if (this.parameters.Length > 2 && this.parameters[2] != null) customErrorMessage = (string)this.parameters[2];
            ValidationEventHandler eventHandler = new ValidationEventHandler(ValidationEventHandler);
            XmlReader xmlSchemaReader = null;
            XmlReaderSettings settings;
            try {
                xmlSchemaReader = XmlReader.Create(new StringReader(xmlSchema));
                settings = new XmlReaderSettings();
                XmlSchema schema = XmlSchema.Read(xmlSchemaReader, eventHandler);
                settings.Schemas.Add(schema);
            } catch (Exception e) {
                throw new Exception("XML Schema reading failed in XmlDocument function. " + customErrorMessage, e);
            }
            settings.ValidationType = ValidationType.Schema;
            settings.ValidationEventHandler += eventHandler;
            settings.CloseInput = true;
            settings.ValidationFlags = XmlSchemaValidationFlags.ReportValidationWarnings | XmlSchemaValidationFlags.ProcessIdentityConstraints | XmlSchemaValidationFlags.ProcessInlineSchema | XmlSchemaValidationFlags.ProcessSchemaLocation;
            XmlReader xmlConfigurationReader = XmlReader.Create(new StringReader(xmlData));
            XmlReader reader = XmlReader.Create(xmlConfigurationReader, settings);
            XDocument document = XDocument.Load(reader);
            if (this.xmlValidationErrors.Count > 0) {
                throw new Exception("Errors validating xml data in \"XmlDocument\" function. " + customErrorMessage + " Following errors were encountered:\r\n- " + String.Join("\r\n- ", this.xmlValidationErrors.ToArray<string>()));
            }
            return document;
        }

        private void ValidationEventHandler(object sender, ValidationEventArgs e) {
            this.xmlValidationErrors.Add(e.Severity + " " + e.Message);
        }

    }
}