﻿using System;
using System.Globalization;

namespace QPR.QPRSuite.NCalc {
    public class DataSetRow : IComparable<DataSetRow>, IEquatable<DataSetRow> {

        private object[] data;
        private DataSet dataset;

        public DataSetRow(object[] data, DataSet dataset) {
            this.data = data;
            this.dataset = dataset;
        }

        public object getCell(int columnIndex) {
            if (columnIndex < 0) throw new Exception("Tried to reference dataset column " + columnIndex + "(first column is number 0)");
            if (this.data.Length <= columnIndex) throw new Exception("Dataset row has " + this.data.Length + " columns; tried to reference column " + columnIndex + " (first column is number 0)");
            return this.data[columnIndex];
        }

        public object getCell(string columnName) {
            int columnIndex = this.dataset.getColumnNumber(columnName);
            if (columnIndex == -1) throw new Exception("Column name \"" + columnName + "\" was not found.");
            return this.getCell(columnIndex);
        }

        public object[] getData() {
            return this.data;
        }

        public int numberOfColumns() {
            return this.dataset.columnsCount();
        }

        public string getColumnName(int index) {
            return this.dataset.getColumnName(index);
        }

        public int getColumnNumber(string columnName) {
            return this.dataset.getColumnNumber(columnName);
        }

        public bool Equals(DataSetRow other) {
            if (other == null || this.data.Length != other.data.Length) return false;
            for (int i = 0; i < this.data.Length; i++) {
                if (compareObjects(this.data[i], other.data[i]) != 0) return false;
            }
            return true;
        }

        public override int GetHashCode() {
            int hashCode = 0;
            for (int i = 0; i < this.data.Length; i++) {
                hashCode = hashCode ^ this.data[i].GetHashCode();
            }
            return hashCode;
        }

        public void copyRow(DataSet targetDataSet) {
            object[] newData = new object[this.data.Length];
            this.data.CopyTo(newData, 0);
            targetDataSet.addRow(newData);
        }

        public int CompareTo(DataSetRow other) {
            for (int i = 0; i < this.dataset.columnsCount(); i++) {
                string columnName = this.dataset.getColumnName(i);
                int cellComparisonResult = DataSetRow.compareObjects(this.data[i], other.data[i]);
                if (cellComparisonResult != 0) return cellComparisonResult;
            }
            return 0;
        }

        public static int compareObjects(object sortValueA, object sortValueB) {
            if (sortValueA == null && sortValueB == null) {
                return 0;
            } else if (sortValueA == null) {
                return -1;
            } else if (sortValueB == null) {
                return 1;
            } else if (!Object.ReferenceEquals(sortValueA.GetType(), sortValueB.GetType())) {
                throw new Exception("Values evaluated to different types \"" + sortValueA.GetType() + "\" and \"" + sortValueB.GetType() + "\"");
            } else if (sortValueA is string) {
                return ((string)sortValueA).CompareTo((string)sortValueB);
            } else if (sortValueA is int || sortValueA is Int16 || sortValueA is Int32 || sortValueA is Int64) {
                return Convert.ToInt64(sortValueA, CultureInfo.InvariantCulture).CompareTo(Convert.ToInt64(sortValueB, CultureInfo.InvariantCulture));
            } else if (sortValueA is decimal) {
                return Convert.ToDecimal(sortValueA, CultureInfo.InvariantCulture).CompareTo(Convert.ToDecimal(sortValueB, CultureInfo.InvariantCulture));
            } else if (sortValueA is double) {
                return Convert.ToDouble(sortValueA, CultureInfo.InvariantCulture).CompareTo(Convert.ToDouble(sortValueB, CultureInfo.InvariantCulture));
            } else if (sortValueA is DateTime) {
                return ((DateTime)sortValueA).CompareTo((DateTime)sortValueB);
            } else {
                return 0;
            }
        }

    }
}
